#! /usr/bin/perl
# Work Time Follower results to csv file
# (c) jan@jan-trippler.de 2016
# Tool to generate csv file with daily cumulated work
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

use strict;
use warnings;
use utf8;
use Cwd qw(abs_path);
use File::Basename;

my $bin_path  = abs_path(dirname($0));
my @show = <DATA>;
print STDERR @show, "\n";
close DATA;

if (defined $ARGV[0] && $ARGV[0] eq '-gpl') {
  open GPL, "$bin_path/gpl-2.0.txt" or die "can't open gpl-2.0.txt\n";
  my @gpl = <GPL>;
  print @gpl;
  close GPL;
  exit 0;
}

# check arguments
die "usage: $0 filename\n" unless $ARGV[0];

# variables
my $granularity = 15; # how much minutes to calculate at least
my $min2next    = 5;  # when to add $granularity to time
my %daily_work  = ();

# we calculate in seconds
$granularity *= 60;
$min2next    *= 60;

# open and read wtv file
open REP, $ARGV[0] or die "can't open $ARGV[0] ($!)\n";
my @inp_arr = <REP>;
close REP;

for (my $i = 1; $i<=$#inp_arr; $i++) {
  $_ = $inp_arr[$i];
  s/\r?\n//;
  # day|start_time|end_time|duration|job|activity
  my ($day, $start_time, $end_time, $duration, $job, $activity) = split /\|/;
  $daily_work{$day} = {} unless defined $daily_work{$day};
  my $day_ref = $daily_work{$day};
  $day_ref->{$job} = ['', 0] unless defined $day_ref->{$job};
  $day_ref->{$job}[0] .= $activity . '; ' if $activity;
  my ($hrs, $mins, $secs) = split /:/, $duration;
  my $tm = $hrs * 3600 + $mins * 60 + $secs;
  $day_ref->{$job}[1] += $tm;
}

# calculate result and print on STDOUT
foreach my $day (sort keys %daily_work) {
  my $day_ref = $daily_work{$day};
  print "Tag|Job|Zeit|Kommentar\n";
  foreach my $job (sort keys %$day_ref) {
    my $activity = $day_ref->{$job}[0];
    my $tm = $day_ref->{$job}[1];
    my $gran_tm = int($tm / $granularity) * $granularity;
    my $diff_tm = $tm - $gran_tm;
    $gran_tm += $granularity if $diff_tm >= $min2next;
    my $hrs = $gran_tm / 3600;
    printf "%s|%s|%-4.2f|%s\n", $day, $job, $hrs, $activity
      unless sprintf("%-4.2f", $hrs) eq '0.00';
  }
}

exit 0;

__DATA__

wtf2csv.pl version 0.1, Copyright (C) 2016 Jan Trippler
wtf2csv.pl comes with ABSOLUTELY NO WARRANTY. This is free 
software, and you are welcome to redistribute it under certain
conditions; type `wtf2csv.pl -gpl' or have a look at the file 
gpl-2.0.txt for details.
